import cadquery as cq
from cadquery import exporters

from modelo import mounting_pillar_positions, shell_t, width
from utils import hex_vents, punch_hole, extrude_shape2

# Dimensions for countersunk M4 screws
m4_top = 9
m4_bottom = 4


lip_thickness = 1.5
# Position of pillar + shell_t + pillar "radius" + lip
height = max([y for _, y in mounting_pillar_positions]) + 6 + shell_t + lip_thickness
thickness = shell_t
front_lip = 8


def model():
    # Create the basic shape of the case lid
    model = (
        cq.Workplane("XY")
        # Hollow box
        .box(width, height, thickness)
        .edges("|Z and >Y")
        .fillet(2)
    )

    # Make many holes
    vent = hex_vents(size=6, width=width * 0.9, height=height * 0.9)[0]
    model = punch_hole(
        model=model,
        face=">Z",
        w=width,
        h=height,
        x_offset=0.05 * width,
        y_offset=0.05 * height,
        hole=vent,
        depth=thickness,
    )

    # Add screw holes
    for position in mounting_pillar_positions:
        model = (
            model.faces(">Z")
            .workplane(centerOption="CenterOfBoundBox")
            .center(-width / 2 + position[0], height / 2 - position[1] - shell_t)
            .placeSketch(cq.Sketch().circle(m4_top / 2 + 1.5))
            .extrude(-thickness)
            .faces(">Z")
            .workplane(centerOption="CenterOfBoundBox")
            .center(-width / 2 + position[0], height / 2 - position[1] - shell_t)
            .cskHole(m4_bottom, m4_top, 82, depth=None)
        )

    # Add front lip

    model = (
        model.faces(">Z")
        .workplane(centerOption="CenterOfBoundBox")
        .center(0, -height / 2 + lip_thickness / 2)
        .placeSketch(cq.Sketch().trapezoid(width - 2 * shell_t, lip_thickness, 90))
        .extrude(-front_lip - thickness)
    )

    return model


def decorative_cover():
    # A decorative thingie to cover the ugly seam in the middle
    model = cq.Workplane("XY").box(10, height, 1).edges("|Z").fillet(1)
    vent = hex_vents(size=6, width=width * 0.9, height=height * 0.9, density=0.775)[0]

    model = extrude_shape2(
        model=model,
        face=">Z",
        w=width,
        h=height,
        x_offset=0.05 * width,
        y_offset=0.05 * height,
        hole=vent,
        depth=3,
    )
    return model


if __name__ == "__main__":
    model = model()
    exporters.export(model, "lid.stl")

    cover = decorative_cover()
    exporters.export(cover, "lid_cover.stl")

    exporters.export(
        model,
        "lid.svg",
        opt={
            "projectionDir": (0, 0, 1),
        },
    )

    exporters.export(
        model.faces(">X").workplane(offset=-width / 2).split(keepTop=True),
        "right_side_lid.stl",
    )
    exporters.export(
        model.faces(">X").workplane(offset=-width / 2).split(keepBottom=True),
        "left_side_lid.stl",
    )
