import cadquery as cq
from cadquery import exporters
from cq_warehouse.drafting import Draft

import audio_plug
import battery_holder
import hdmi_out
import keyboard
import screen_pillars
import usb_hub
import zero_holder as cpu_holder

# Base for the notebook. Basically a kbd base that extends back
# as much as possible

# Thickness of the outer material
shell_t = 3

# Size of the kbd board
kbd_height = 95.5
kbd_width = 305

# Size of the whole object
width = kbd_width + 2 * shell_t
height = 159
thickness = 27 + shell_t  # 27 inside

# Insert Positions
ti_radius = 2.35
ti_depth = 6.25

# These are placed where convenient, and are used to join the top and bottom
# parts of the case.
# Measured from top-left corner OUTSIDE
mounting_pillar_positions = [
    (6, 6),
    (6, 48),
    (120, 6),
    (170, 6),
    (width - 6, 6),
    (width - 6, 30),
    (120, 48),
    (170, 48),
]
screen_pillars.init(mounting_pillar_positions, thickness - shell_t)

# Offset for the USB port from back-left corner
# of the case to left side of the hub
usb_offset_x = width - audio_plug.item_w - usb_hub.item_w


# CPU holder position from back-left corner of the case
cpu_offset_x = 177
cpu_offset_y = 2

# Battery holder position from back-left corner of the case
battery_offset_x = 15
battery_offset_y = 3


def model():
    # Create the basic shape of the case bottom.
    model = (
        cq.Workplane("XY")
        .workplane(offset=thickness / 2)
        .tag("mid_height")
        # Hollow box
        .box(width, height, thickness)
        .edges("|Z")
        .fillet(2)
        .faces(">Z")
        .shell(-shell_t)
    )

    # Now the basic box shape is in place, start adding things
    # and cutting holes.

    model = usb_hub.add(
        model=model,
        width=width,
        height=height,
        thickness=thickness,
        bottom_face="<Z",
        back_face=">Y",
        offset_x=usb_offset_x,
        offset_y=0,
        shell_t=shell_t,
    )

    # Hole for audio in right side
    model = audio_plug.add(
        model=model,
        width=width,
        height=height,
        thickness=thickness,
        offset_x=width - audio_plug.item_w,
        offset_y=40,
        bottom_face="<Z",
        back_face=">X",
        shell_t=shell_t,
    )

    # Hole for HDMI out in the back
    model = hdmi_out.add(
        model=model,
        width=width,
        height=height,
        thickness=thickness,
        offset_x=138,
        offset_y=0,
        bottom_face=None,
        back_face=">Y",
        shell_t=shell_t,
    )

    model = cpu_holder.add(
        model=model,
        width=width,
        height=height,
        thickness=thickness,
        offset_x=cpu_offset_x,
        offset_y=cpu_offset_y,
        bottom_face="<Z",
        back_face=None,  # Not exposing the holes
        shell_t=shell_t,
    )

    # This adds all the holes and extrusions for the battery system
    model = battery_holder.add(
        model=model,
        width=width,
        height=height,
        thickness=thickness,
        offset_x=battery_offset_x,
        offset_y=battery_offset_y,
        bottom_face="<Z",
        back_face=">Y",
        shell_t=shell_t,
    )

    model = screen_pillars.add(
        model=model,
        width=width,
        height=height,
        thickness=thickness,
        offset_x=0,
        offset_y=0,
        bottom_face="<Z",
        back_face=None,
        shell_t=shell_t,
    )

    model = keyboard.add(
        model=model,
        width=width,
        height=height,
        thickness=thickness,
        bottom_face="<Z",
        back_face=None,
        offset_x=shell_t,
        offset_y=kbd_height + shell_t,
        shell_t=shell_t,
    )

    return model


if __name__ == "__main__":
    model = model()

    left_cutout = cq.Sketch().polygon(
        [(0, 0), (width / 2, 0), (width / 2, -height), (0, -height), (0, 0)],
        mode="a",
    )

    right_side = (
        model.faces("<Z")
        .workplaneFromTagged("mid_height")
        .transformed(offset=cq.Vector(0, 0, -thickness / 2))
        .center(-width / 2, height / 2)
        .placeSketch(left_cutout)
        .cutBlind(100)
    )

    exporters.export(right_side, "right_side.stl")

    right_cutout = cq.Sketch().polygon(
        [
            (width / 2, 0),
            (width, 0),
            (width, -height),
            (width / 2, -height),
            (width / 2, 0),
        ],
        mode="a",
    )

    left_side = (
        model.faces("<Z")
        .workplaneFromTagged("mid_height")
        .transformed(offset=cq.Vector(0, 0, -thickness / 2))
        .center(-width / 2, height / 2)
        .placeSketch(right_cutout)
        .cutBlind(100)
    )
    exporters.export(left_side, "left_side.stl")

    draft = Draft(decimal_precision=1)
    dimensions = []
    dimensions.append(
        draft.extension_line(
            object_edge=[
                cq.Vertex.makeVertex(-width / 2, -height / 2, 0),
                cq.Vertex.makeVertex(width / 2, -height / 2, 0),
            ],
            offset=10.0,
        )
    )
    dimensions.append(
        draft.extension_line(
            object_edge=[
                cq.Vertex.makeVertex(width / 2, -height / 2, 0),
                cq.Vertex.makeVertex(width / 2, height / 2, 0),
            ],
            offset=10.0,
        )
    )

    exporters.export(model, "model.stl")

    for d in dimensions[1:]:
        dimensions[0].add(d.toCompound())
    dimensions[0].add(model)

    exporters.export(
        dimensions[0].toCompound(),
        "model.svg",
        opt={
            "projectionDir": (0, 0, 1),
            "strokeWidth": 0.3,
        },
    )
